==> ui-18.0.1.tgz: package/components/cascader/List.js <==
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
import { createElement as _createElement } from "react";
import { jsx as _jsx, Fragment as _Fragment, jsxs as _jsxs } from "react/jsx-runtime";
import { isUndefined } from 'lodash';
import React, { useImperativeHandle, useMemo, useRef } from 'react';
import { useEventCallback } from '@react-devui/hooks';
import { LoadingOutlined, RightOutlined } from '@react-devui/icons';
import { getClassName } from '@react-devui/utils';
import { DCheckbox } from '../checkbox';
import { usePrefixConfig, useTranslation } from '../root';
import { DVirtualScroll } from '../virtual-scroll';
function List(props, ref) {
    const { dGetItemId, dList, dFocusItem, dCustomItem, dMultiple, dFocusVisible, dRoot, onFocusChange, onClickItem } = props, restProps = __rest(props, ["dGetItemId", "dList", "dFocusItem", "dCustomItem", "dMultiple", "dFocusVisible", "dRoot", "onFocusChange", "onClickItem"]);
    //#region Context
    const dPrefix = usePrefixConfig();
    //#endregion
    //#region Ref
    const vsRef = useRef(null);
    const comboboxKeyDownRef = useRef(null);
    //#endregion
    const [t] = useTranslation();
    const isFocus = dFocusItem && dList.findIndex((node) => node.id === dFocusItem.id) !== -1;
    const inFocusNode = (() => {
        if (dFocusItem) {
            for (const node of dList) {
                if (dFocusItem.id === node.id) {
                    return node;
                }
                let _node = dFocusItem;
                while (_node.parent) {
                    _node = _node.parent;
                    if (_node.id === node.id) {
                        return node;
                    }
                }
            }
        }
    })();
    const shouldInitFocus = dRoot && isUndefined(dFocusItem);
    const handleKeyDown = useEventCallback((key) => {
        var _a, _b, _c, _d, _e, _f, _g, _h, _j;
        const focusNode = (node) => {
            if (node) {
                onFocusChange(node);
            }
        };
        if (shouldInitFocus) {
            switch (key) {
                case 'next':
                    focusNode((_a = vsRef.current) === null || _a === void 0 ? void 0 : _a.scrollToStart());
                    break;
                case 'prev':
                    focusNode((_b = vsRef.current) === null || _b === void 0 ? void 0 : _b.scrollToEnd());
                    break;
                case 'first':
                    focusNode((_c = vsRef.current) === null || _c === void 0 ? void 0 : _c.scrollToStart());
                    break;
                case 'last':
                    focusNode((_d = vsRef.current) === null || _d === void 0 ? void 0 : _d.scrollToEnd());
                    break;
                default:
                    break;
            }
        }
        else if (isFocus) {
            switch (key) {
                case 'next':
                    focusNode((_e = vsRef.current) === null || _e === void 0 ? void 0 : _e.scrollToStep(1));
                    break;
                case 'prev':
                    focusNode((_f = vsRef.current) === null || _f === void 0 ? void 0 : _f.scrollToStep(-1));
                    break;
                case 'first':
                    focusNode((_g = vsRef.current) === null || _g === void 0 ? void 0 : _g.scrollToStart());
                    break;
                case 'last':
                    focusNode((_h = vsRef.current) === null || _h === void 0 ? void 0 : _h.scrollToEnd());
                    break;
                case 'prev-level':
                    if (inFocusNode && inFocusNode.parent) {
                        onFocusChange(inFocusNode.parent);
                    }
                    break;
                case 'next-level':
                    if (inFocusNode && inFocusNode.children) {
                        for (const node of inFocusNode.children) {
                            if (node.enabled) {
                                onFocusChange(node);
                                break;
                            }
                        }
                    }
                    break;
                case 'click':
                    if (inFocusNode) {
                        onClickItem(inFocusNode);
                    }
                    break;
                default:
                    break;
            }
        }
        else {
            (_j = comboboxKeyDownRef.current) === null || _j === void 0 ? void 0 : _j.call(comboboxKeyDownRef, key);
        }
    });
    useImperativeHandle(ref, () => handleKeyDown, [handleKeyDown]);
    const vsPerformance = useMemo(() => ({
        dList,
        dItemSize: 32,
        dItemKey: (item) => item.id,
        dFocusable: (item) => item.enabled,
    }), [dList]);
    return (_jsxs(_Fragment, { children: [_jsx(DVirtualScroll, Object.assign({}, vsPerformance, { ref: vsRef, dFillNode: _jsx("li", {}), dItemRender: (item, index, { aria }) => (_createElement("li", Object.assign({}, aria, { key: item.id, id: dGetItemId(item.id), className: getClassName(`${dPrefix}cascader__option`, {
                        'is-focus': item.id === (inFocusNode === null || inFocusNode === void 0 ? void 0 : inFocusNode.id),
                        'is-selected': !dMultiple && item.checked,
                        'is-disabled': item.disabled,
                    }), title: item.origin.label, role: "option", "aria-selected": item.checked, "aria-disabled": item.disabled, onClick: () => {
                        onFocusChange(item);
                        if (!dMultiple || item.isLeaf) {
                            onClickItem(item);
                        }
                    } }),
                    dFocusVisible && item.id === (dFocusItem === null || dFocusItem === void 0 ? void 0 : dFocusItem.id) && _jsx("div", { className: `${dPrefix}focus-outline` }),
                    dMultiple && (_jsx(DCheckbox, { onClick: (e) => {
                            e.stopPropagation();
                            onFocusChange(item);
                            onClickItem(item);
                        }, dModel: item.checked, dDisabled: item.disabled, dIndeterminate: item.indeterminate })),
                    _jsx("div", Object.assign({ className: `${dPrefix}cascader__option-content` }, { children: dCustomItem ? dCustomItem(item.origin) : item.origin.label })),
                    !item.isLeaf && (_jsx("div", Object.assign({ className: `${dPrefix}cascader__option-icon` }, { children: item.origin.loading ? _jsx(LoadingOutlined, { dSpin: true }) : _jsx(RightOutlined, {}) }))))), dFocusItem: inFocusNode, dSize: 264, dPadding: 4 }, { children: ({ render, vsList }) => render(
                // eslint-disable-next-line jsx-a11y/aria-activedescendant-has-tabindex
                _jsx("ul", Object.assign({}, restProps, { className: getClassName(restProps.className, `${dPrefix}cascader__list`), tabIndex: -1, role: "listbox", "aria-multiselectable": dMultiple, "aria-activedescendant": dRoot && !isUndefined(dFocusItem) ? dGetItemId(dFocusItem.id) : undefined }, { children: dList.length === 0 ? (_jsx("li", Object.assign({ className: `${dPrefix}cascader__empty` }, { children: _jsx("div", Object.assign({ className: `${dPrefix}cascader__option-content` }, { children: t('No Data') })) }))) : (vsList) }))) })), inFocusNode && !inFocusNode.origin.loading && inFocusNode.children && (_jsx(DList, Object.assign({}, props, { ref: comboboxKeyDownRef, id: undefined, dList: inFocusNode.children, dRoot: false })))] }));
}
export const DList = React.forwardRef(List);

==> ui-18.3.5.tgz: package/components/cascader/List.js <==
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
import { createElement as _createElement } from "react";
import { jsx as _jsx, Fragment as _Fragment, jsxs as _jsxs } from "react/jsx-runtime";
import { isUndefined } from 'lodash';
import React, { useImperativeHandle, useMemo, useRef } from 'react';
import { useEventCallback } from '@react-devui/hooks';
import { LoadingOutlined, RightOutlined } from '@react-devui/icons';
import { getClassName } from '@react-devui/utils';
import { DCheckbox } from '../checkbox';
import { usePrefixConfig, useTranslation } from '../root';
import { DVirtualScroll } from '../virtual-scroll';
function List(props, ref) {
    const { dGetItemId, dList, dFocusItem, dCustomItem, dMultiple, dVirtual, dFocusVisible, dRoot, onFocusChange, onClickItem } = props, restProps = __rest(props, ["dGetItemId", "dList", "dFocusItem", "dCustomItem", "dMultiple", "dVirtual", "dFocusVisible", "dRoot", "onFocusChange", "onClickItem"]);
    //#region Context
    const dPrefix = usePrefixConfig();
    //#endregion
    //#region Ref
    const vsRef = useRef(null);
    const comboboxKeyDownRef = useRef(null);
    //#endregion
    const [t] = useTranslation();
    const isFocus = dFocusItem && dList.findIndex((node) => node.id === dFocusItem.id) !== -1;
    const inFocusNode = (() => {
        if (dFocusItem) {
            for (const node of dList) {
                if (dFocusItem.id === node.id) {
                    return node;
                }
                let _node = dFocusItem;
                while (_node.parent) {
                    _node = _node.parent;
                    if (_node.id === node.id) {
                        return node;
                    }
                }
            }
        }
    })();
    const shouldInitFocus = dRoot && isUndefined(dFocusItem);
    const handleKeyDown = useEventCallback((key) => {
        var _a, _b, _c, _d, _e, _f, _g, _h, _j;
        const focusNode = (node) => {
            if (node) {
                onFocusChange(node);
            }
        };
        if (shouldInitFocus) {
            switch (key) {
                case 'next':
                    focusNode((_a = vsRef.current) === null || _a === void 0 ? void 0 : _a.scrollToStart());
                    break;
                case 'prev':
                    focusNode((_b = vsRef.current) === null || _b === void 0 ? void 0 : _b.scrollToEnd());
                    break;
                case 'first':
                    focusNode((_c = vsRef.current) === null || _c === void 0 ? void 0 : _c.scrollToStart());
                    break;
                case 'last':
                    focusNode((_d = vsRef.current) === null || _d === void 0 ? void 0 : _d.scrollToEnd());
                    break;
                default:
                    break;
            }
        }
        else if (isFocus) {
            switch (key) {
                case 'next':
                    focusNode((_e = vsRef.current) === null || _e === void 0 ? void 0 : _e.scrollToStep(1));
                    break;
                case 'prev':
                    focusNode((_f = vsRef.current) === null || _f === void 0 ? void 0 : _f.scrollToStep(-1));
                    break;
                case 'first':
                    focusNode((_g = vsRef.current) === null || _g === void 0 ? void 0 : _g.scrollToStart());
                    break;
                case 'last':
                    focusNode((_h = vsRef.current) === null || _h === void 0 ? void 0 : _h.scrollToEnd());
                    break;
                case 'prev-level':
                    if (inFocusNode && inFocusNode.parent) {
                        onFocusChange(inFocusNode.parent);
                    }
                    break;
                case 'next-level':
                    if (inFocusNode && inFocusNode.children) {
                        for (const node of inFocusNode.children) {
                            if (node.enabled) {
                                onFocusChange(node);
                                break;
                            }
                        }
                    }
                    break;
                case 'click':
                    if (inFocusNode) {
                        onClickItem(inFocusNode);
                    }
                    break;
                default:
                    break;
            }
        }
        else {
            (_j = comboboxKeyDownRef.current) === null || _j === void 0 ? void 0 : _j.call(comboboxKeyDownRef, key);
        }
    });
    useImperativeHandle(ref, () => handleKeyDown, [handleKeyDown]);
    const vsPerformance = useMemo(() => ({
        dList,
        dItemSize: 32,
        dItemKey: (item) => item.id,
        dFocusable: (item) => item.enabled,
    }), [dList]);
    return (_jsxs(_Fragment, { children: [_jsx(DVirtualScroll, Object.assign({}, vsPerformance, { ref: vsRef, dFillNode: _jsx("li", {}), dItemRender: (item, index, { aria }) => (_createElement("li", Object.assign({}, aria, { key: item.id, id: dGetItemId(item.id), className: getClassName(`${dPrefix}cascader__option`, {
                        'is-focus': item.id === (inFocusNode === null || inFocusNode === void 0 ? void 0 : inFocusNode.id),
                        'is-selected': !dMultiple && item.checked,
                        'is-disabled': item.disabled,
                    }), title: item.origin.label, role: "option", "aria-selected": item.checked, "aria-disabled": item.disabled, onClick: () => {
                        onFocusChange(item);
                        if (!dMultiple || item.isLeaf) {
                            onClickItem(item);
                        }
                    } }),
                    dFocusVisible && item.id === (dFocusItem === null || dFocusItem === void 0 ? void 0 : dFocusItem.id) && _jsx("div", { className: `${dPrefix}focus-outline` }),
                    dMultiple && (_jsx(DCheckbox, { onClick: (e) => {
                            e.stopPropagation();
                            onFocusChange(item);
                            onClickItem(item);
                        }, dModel: item.checked, dDisabled: item.disabled, dIndeterminate: item.indeterminate })),
                    _jsx("div", Object.assign({ className: `${dPrefix}cascader__option-content` }, { children: dCustomItem ? dCustomItem(item.origin) : item.origin.label })),
                    !item.isLeaf && (_jsx("div", Object.assign({ className: `${dPrefix}cascader__option-icon` }, { children: item.origin.loading ? _jsx(LoadingOutlined, { dSpin: true }) : _jsx(RightOutlined, {}) }))))), dFocusItem: inFocusNode, dSize: dVirtual ? 264 : Infinity, dPadding: 4 }, { children: ({ render, vsList }) => render(
                // eslint-disable-next-line jsx-a11y/aria-activedescendant-has-tabindex
                _jsx("ul", Object.assign({}, restProps, { className: getClassName(restProps.className, `${dPrefix}cascader__list`), tabIndex: -1, role: "listbox", "aria-multiselectable": dMultiple, "aria-activedescendant": dRoot && !isUndefined(dFocusItem) ? dGetItemId(dFocusItem.id) : undefined }, { children: dList.length === 0 ? (_jsx("li", Object.assign({ className: `${dPrefix}cascader__empty` }, { children: _jsx("div", Object.assign({ className: `${dPrefix}cascader__option-content` }, { children: t('No Data') })) }))) : (vsList) }))) })), inFocusNode && !inFocusNode.origin.loading && inFocusNode.children && (_jsx(DList, Object.assign({}, props, { ref: comboboxKeyDownRef, id: undefined, dList: inFocusNode.children, dRoot: false })))] }));
}
export const DList = React.forwardRef(List);
